import { defineStore } from "pinia";
import { usePatientsStore } from "./patientsStore.js";
import { useFilesStore } from "@/store/filesStore.js";
import axios from "axios";
import getIdFromLink from "@/services/parsers.js";
import { useReminderStore } from "@/store/reminderStore.js";

export const useConsultationsStore = defineStore({
  id: "consultationsStore",
  state: () => ({
    consultations: [],
  }),
  actions: {
    async loadData() {
      if (this.consultations === null || this.consultations.length === 0) {
        await this.fetchConsultations();
      }
    },

    async fetchConsultations() {
      const response = await axios.get(
        import.meta.env.VITE_APP_API_URL + "consultations"
      );

      if (response.data._embedded) {
        let regularVisits;
        let emergencies;
        if (response.data._embedded.regularVisits) {
          regularVisits = response.data._embedded.regularVisits;
        }
        if (response.data._embedded.emergencies) {
          emergencies = response.data._embedded.emergencies;
        }
        let consultations;
        if (regularVisits && emergencies) {
          consultations = regularVisits.concat(emergencies);
        } else if (regularVisits) {
          consultations = regularVisits;
        } else if (emergencies) {
          consultations = emergencies;
        }

        this.consultations = await Promise.all(
          consultations.map(async (consultation) => {
            const selfLink = consultation._links.self.href;
            const patientLink = consultation._links.patient.href;
            return {
              id: getIdFromLink(selfLink),
              patient: await usePatientsStore().getPatient(
                getIdFromLink(patientLink)
              ),
              ...consultation,
            };
          })
        );
      } else {
        this.consultations = [];
      }
    },

    async getConsultations() {
      await this.loadData();
      return this.consultations;
    },

    async getConsultation(id) {
      const consultationData = await axios.get(
        import.meta.env.VITE_APP_API_URL + "consultations/" + id
      );
      const consultation = consultationData.data;

      consultation.symptoms = [];
      consultation.files = [];
      try {
        if (consultation.type === "Emergency") {
          const symptomsData = await axios.get(
            import.meta.env.VITE_APP_API_URL +
              "consultations/" +
              id +
              "/symptoms"
          );
          if (symptomsData.data._embedded) {
            consultation.symptoms = symptomsData.data._embedded.symptoms;
            consultation.symptoms.map((symptom) => {
              symptom.id = getIdFromLink(symptom._links.self.href);
              return symptom;
            });
          }
        }
      } catch (error) {}

      const filesData = await axios.get(
        import.meta.env.VITE_APP_API_URL + "consultations/" + id + "/files"
      );

      if (filesData.data._embedded) {
        consultation.files = filesData.data._embedded.files;
      }

      const selfLink = consultation._links.self.href;
      const patientLink = consultation._links.patient.href;
      consultation.id = getIdFromLink(selfLink);

      consultation.patient = await usePatientsStore().getPatient(
        getIdFromLink(patientLink)
      );
      return consultation;
    },

    async getConsultationCopy(id) {
      const consultation = await this.getConsultation(id);
      return {
        ...consultation,
      };
    },

    async addConsultation(consultationToAdd) {
      const consultation = { ...consultationToAdd };
      consultation.files = [];
      consultation.patient =
        import.meta.env.VITE_APP_API_URL +
        "patients/" +
        consultation.patient.id;

      if (consultation.blobFiles) {
        for (const file of consultation.blobFiles) {
          const response = await useFilesStore().uploadFile(file.rawFile);
          consultation.files.push(response.data._links.self.href);
        }
      }

      consultation.symptoms = consultation.symptoms.map((symptom) => {
        return import.meta.env.VITE_APP_API_URL + "symptoms/" + symptom.id;
      });

      const consultationData = await axios.post(
        import.meta.env.VITE_APP_API_URL + "consultations",
        consultation
      );
      const addedConsultation = consultationData.data;
      const selfLink = addedConsultation._links.self.href;
      const patientLink = addedConsultation._links.patient.href;
      addedConsultation.id = getIdFromLink(selfLink);
      addedConsultation.patient = await usePatientsStore().getPatient(
        getIdFromLink(patientLink)
      );
      this.consultations.push(addedConsultation);

      useReminderStore().fetchReminder();
      return consultation;
    },

    async editConsultation(editedConsultation) {
      const consultation = { ...editedConsultation };
      consultation.patient =
        import.meta.env.VITE_APP_API_URL +
        "patients/" +
        consultation.patient.id;

      consultation.symptoms = consultation.symptoms.map((symptom) => {
        return import.meta.env.VITE_APP_API_URL + "symptoms/" + symptom.id;
      });
      const consultationData = await axios.put(
        import.meta.env.VITE_APP_API_URL + "consultations/" + consultation.id,
        consultation
      );
      const newConsultation = consultationData.data;
      const selfLink = newConsultation._links.self.href;
      const patientLink = newConsultation._links.patient.href;
      newConsultation.id = getIdFromLink(selfLink);
      newConsultation.patient = await usePatientsStore().getPatient(
        getIdFromLink(patientLink)
      );

      useReminderStore().fetchReminder();

      this.consultations = this.consultations.map((consultation) => {
        if (consultation.id === newConsultation.id) {
          return newConsultation;
        }
        return consultation;
      });
    },

    async deleteConsultation(id) {
      await axios.delete(
        import.meta.env.VITE_APP_API_URL + "consultations/" + id
      );

      useReminderStore().fetchReminder();

      this.consultations = this.consultations.filter(
        (consultation) => consultation.id !== id
      );
    },

    async uploadFile(file, consultation) {
      const fileData = await useFilesStore().uploadFile(file.rawFile);
      const fileUploaded = fileData.data._links.self.href;
      await axios.post(
        import.meta.env.VITE_APP_API_URL +
          "consultations/" +
          consultation.id +
          "/files",
        { file: fileUploaded }
      );

      await consultation.files.push(fileData.data);
    },

    async deleteFile(file, consultation) {
      const id = getIdFromLink(file._links.self.href);
      await axios.delete(
        import.meta.env.VITE_APP_API_URL +
          "consultations/" +
          consultation.id +
          "/files/" +
          id
      );
      consultation.files = consultation.files.filter(
        (f) => file._links.self.href !== f._links.self.href
      );
    },
  },
});
