package es.mdef.mipediatra.assemblers;

import es.mdef.mipediatra.constants.ConsultationType;
import es.mdef.mipediatra.controllers.ConsultationsController;
import es.mdef.mipediatra.controllers.PatientsController;
import es.mdef.mipediatra.entities.*;
import es.mdef.mipediatra.exceptions.InvalidConversionException;
import es.mdef.mipediatra.models.consultation.*;
import es.mdef.mipediatra.repositories.PatientRepository;
import es.mdef.mipediatra.services.AssemblerService;
import es.mdef.mipediatra.services.impl.MeasurementService;
import org.springframework.stereotype.Component;

import java.util.HashSet;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@Component
public class ConsultationAssembler extends AssemblerService<ConsultationEntity, ConsultationModel, ConsultationPostModel, ConsultationPutModel, ConsultationListModel> {
    private final MeasurementService measurementService;
    private final PatientRepository patientRepository;

    public ConsultationAssembler(MeasurementService measurementService, PatientRepository patientRepository) {
        super(() -> linkTo(methodOn(ConsultationsController.class).getAll()).withSelfRel());
        this.measurementService = measurementService;
        this.patientRepository = patientRepository;
    }

    @Override
    public ConsultationModel toModel(ConsultationEntity consultation) {
        ConsultationModel model = new ConsultationModel();

        if (consultation instanceof RegularVisitEntity regularVisit) {
            RegularVisitModel regularVisitModel = new RegularVisitModel();
            regularVisitModel.setMeasurement(measurementService.getAssembler().toModel((MeasurementEntity) regularVisit.getMeasurement()));
            model = regularVisitModel;
        } else if (consultation instanceof EmergencyEntity emergency) {
            EmergencyModel emergencyModel = new EmergencyModel();
            emergencyModel.setDiagnosis(emergency.getDiagnosis());
            emergencyModel.setTreatment(emergency.getTreatment());

            model = emergencyModel;
            model.add(linkTo(methodOn(ConsultationsController.class).symptoms(consultation.getId())).withRel("symptoms"));
        }

        model.setComments(consultation.getComments());
        model.setDate(consultation.getDate());
        model.setCenter(consultation.getCenter());
        model.setSpecialist(consultation.getSpecialist());

        model.add(linkTo(methodOn(ConsultationsController.class).getById(consultation.getId())).withSelfRel());
        model.add(linkTo(methodOn(PatientsController.class).getById(((PatientEntity) consultation.getPatient()).getId())).withRel("patient"));
        model.add(linkTo(methodOn(ConsultationsController.class).files(((PatientEntity) consultation.getPatient()).getId())).withRel("files"));
        return model;
    }

    @Override
    public ConsultationListModel toListModel(ConsultationEntity consultation) {
        ConsultationListModel model = new ConsultationListModel();

        if (consultation instanceof RegularVisitEntity regularVisit) {
            model = new RegularVisitListModel();
        } else if (consultation instanceof EmergencyEntity emergency) {
            model = new EmergencyListModel();
        }

        model.setSpecialist(consultation.getSpecialist());
        model.setComments(consultation.getComments());
        model.setDate(consultation.getDate());


        model.add(linkTo(methodOn(ConsultationsController.class).getById(consultation.getId())).withSelfRel());
        model.add(linkTo(methodOn(PatientsController.class).getById(((PatientEntity) consultation.getPatient()).getId())).withRel("patient"));
        model.add(linkTo(methodOn(ConsultationsController.class).files(((PatientEntity) consultation.getPatient()).getId())).withRel("files"));
        return model;
    }

    @Override
    public ConsultationEntity toEntity(ConsultationPostModel model) {
        ConsultationEntity consultation = null;

        if (model.getType() == ConsultationType.RegularVisit) {
            RegularVisitEntity regularVisit = new RegularVisitEntity();

            MeasurementEntity measurementEntity = measurementService.getAssembler().toEntity(model.getMeasurement());
            measurementEntity = measurementService.getRepository().save(measurementEntity);
            regularVisit.setMeasurement(measurementEntity);

            consultation = regularVisit;
        } else if (model.getType() == ConsultationType.Emergency) {
            EmergencyEntity emergency = new EmergencyEntity();
            emergency.setDiagnosis(model.getDiagnosis());
            emergency.setTreatment(model.getTreatment());

            emergency.setSymptoms(new HashSet<>(model.getSymptoms()));
            consultation = emergency;
        }

        assert consultation != null;
        consultation.setSpecialist(model.getSpecialist());
        consultation.setComments(model.getComments());
        consultation.setCenter(model.getCenter());
        consultation.setDate(model.getDate());
        model.getPatient().addConsultation(consultation);
        consultation.setPatient(model.getPatient());

        if (model.getFiles() != null) {
            consultation.setFiles(new HashSet<>(model.getFiles()));
        }

        return consultation;
    }

    @Override
    public ConsultationEntity toEntity(ConsultationPutModel model) {
        return null;
    }

    @Override
    public ConsultationEntity update(ConsultationEntity entity, ConsultationPutModel model) {
        ConsultationEntity consultation = null;

        if (entity instanceof RegularVisitEntity regularVisit) {
            if (model.getType() != ConsultationType.RegularVisit) {
                throw new InvalidConversionException("consulta");
            }
            measurementService.update(((MeasurementEntity) regularVisit.getMeasurement()).getId(), model.getMeasurement());
            consultation = regularVisit;
        } else if (entity instanceof EmergencyEntity emergency) {
            if (model.getType() != ConsultationType.Emergency) {
                throw new InvalidConversionException("consulta");
            }
            emergency.setDiagnosis(model.getDiagnosis());
            emergency.setTreatment(model.getTreatment());
            emergency.setSymptoms(new HashSet<>(model.getSymptoms()));
            consultation = emergency;
        }

        assert consultation != null;
        consultation.setSpecialist(model.getSpecialist());
        consultation.setComments(model.getComments());
        consultation.setCenter(model.getCenter());
        consultation.setDate(model.getDate());

        PatientEntity oldPatient = (PatientEntity) consultation.getPatient();
        consultation.setPatient(model.getPatient());

        return consultation;
    }
}