package es.mdef.mipediatra.services.impl;

import es.mdef.mipediatra.assemblers.ConsultationAssembler;
import es.mdef.mipediatra.assemblers.FileAssembler;
import es.mdef.mipediatra.assemblers.SymptomAssembler;
import es.mdef.mipediatra.entities.*;
import es.mdef.mipediatra.exceptions.BadRelationException;
import es.mdef.mipediatra.exceptions.ConsultationNotAnEmergencyException;
import es.mdef.mipediatra.exceptions.EntityNotFoundException;
import es.mdef.mipediatra.models.consultation.*;
import es.mdef.mipediatra.models.file.FileListModel;
import es.mdef.mipediatra.models.symptom.SymptomsListModel;
import es.mdef.mipediatra.repositories.ConsultationRepository;
import es.mdef.mipediatra.repositories.RegularVisitRepository;
import es.mdef.mipediatra.services.AssemblerService;
import es.mdef.mipediatra.services.CrudService;
import es.mdef.mipediatralib.interfaces.IConsultation;
import es.mdef.mipediatralib.interfaces.IFile;
import es.mdef.mipediatralib.interfaces.IPatient;
import es.mdef.mipediatralib.interfaces.ISymptom;
import jakarta.transaction.Transactional;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.hateoas.CollectionModel;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.Set;
import java.util.stream.Collectors;

@Service
public class ConsultationService implements CrudService<ConsultationEntity, ConsultationModel, ConsultationPostModel, ConsultationPutModel, ConsultationListModel> {

    private final ConsultationRepository consultationRepository;
    private final ConsultationAssembler consultationAssembler;
    private final SymptomAssembler symptomAssembler;
    private final FileAssembler fileAssembler;
    private final FileStorageService fileStorageService;
    private final RegularVisitRepository regularVisitRepository;

    public ConsultationService(ConsultationRepository consultationRepository, ConsultationAssembler consultationAssembler, SymptomAssembler symptomAssembler, FileAssembler fileAssembler, FileStorageService fileStorageService, RegularVisitRepository regularVisitRepository) {
        this.consultationRepository = consultationRepository;
        this.consultationAssembler = consultationAssembler;
        this.symptomAssembler = symptomAssembler;
        this.fileAssembler = fileAssembler;
        this.fileStorageService = fileStorageService;
        this.regularVisitRepository = regularVisitRepository;
    }

    @Override
    public String getEntityName() {
        return "consultation";
    }

    @Override
    public JpaRepository<ConsultationEntity, Long> getRepository() {
        return consultationRepository;
    }

    @Override
    public AssemblerService<ConsultationEntity, ConsultationModel, ConsultationPostModel, ConsultationPutModel, ConsultationListModel> getAssembler() {
        return consultationAssembler;
    }

    public CollectionModel<SymptomsListModel> getSymptoms(long id) {
        ConsultationEntity consultation = consultationRepository.findById(id)
                .orElseThrow(() -> new EntityNotFoundException(id, "consultation"));

        Set<SymptomEntity> symptoms = new HashSet<>();
        if (!(consultation instanceof EmergencyEntity emergency)) {
            throw new ConsultationNotAnEmergencyException(id);
        } else {

            Set<ISymptom> iSymptoms = emergency.getSymptoms();
            for (ISymptom iSymptom : iSymptoms) {
                if (iSymptom instanceof SymptomEntity entity) {
                    symptoms.add(entity);
                }
            }
        }
        return symptomAssembler.toCollectionModel(symptoms);
    }

    @Transactional
    public CollectionModel<FileListModel> getFiles(long id) {
        ConsultationEntity consultation = consultationRepository.findById(id)
                .orElseThrow(() -> new EntityNotFoundException(id, "consultation"));

        Set<FileEntity> fileEntities = new HashSet<>();
        for (IFile iFile : consultation.getFiles()) {
            if (iFile instanceof FileEntity) {
                fileEntities.add((FileEntity) iFile);
            }
        }
        return fileAssembler.toCollectionModel(fileEntities);
    }

    @Transactional
    public void removeFile(long id, long fileId) {
        ConsultationEntity consultation = consultationRepository.findById(id)
                .orElseThrow(() -> new EntityNotFoundException(id, "consultation"));

        FileEntity fileEntity = fileStorageService.getFileEntity(fileId);
        if (consultation.getFiles().contains(fileEntity)) {
            consultation.getFiles().remove(fileEntity);
            fileStorageService.deleteFile(fileEntity);
        } else {
            throw new BadRelationException("file", fileId, "consultation", id);
        }

        consultationRepository.save(consultation);
    }

    @Transactional
    public void addFile(long id, FileEntity file) {
        ConsultationEntity consultation = consultationRepository.findById(id)
                .orElseThrow(() -> new EntityNotFoundException(id, "consultation"));
        if (file != null) {
            consultation.getFiles().add(file);
        } else {
            throw new EntityNotFoundException(null, "file");
        }

        consultationRepository.save(consultation);
    }

    @Transactional
    public Set<IConsultation> getConsultationsByPatient(IPatient patient) {
        return consultationRepository.findConsultationsByPatientOrderByDateDesc(patient);
    }

    @Transactional
    public Set<EmergencyEntity> getEmergenciesByPatient(IPatient patient) {
        Set<IConsultation> consultations = getConsultationsByPatient(patient);

        return consultations.stream()
                .filter(EmergencyEntity.class::isInstance)
                .map(EmergencyEntity.class::cast)
                .collect(Collectors.toSet());
    }

    @Transactional
    public RegularVisitModel getRegularVisitByMeasurement(MeasurementEntity measurement) {
        return (RegularVisitModel) consultationAssembler.toModel(regularVisitRepository
                .findByMeasurement(measurement)
                .orElseThrow(() -> new EntityNotFoundException(null, "regular visit")));
    }
}