package es.mdef.mipediatra.services.impl;

import es.mdef.mipediatra.assemblers.PatientAssembler;
import es.mdef.mipediatra.assemblers.SymptomAssembler;
import es.mdef.mipediatra.controllers.PatientsController;
import es.mdef.mipediatra.entities.*;
import es.mdef.mipediatra.exceptions.EntityNotFoundException;
import es.mdef.mipediatra.exceptions.PatientNotAPersonException;
import es.mdef.mipediatra.models.patient.PatientListModel;
import es.mdef.mipediatra.models.patient.PatientModel;
import es.mdef.mipediatra.models.patient.PatientPostModel;
import es.mdef.mipediatra.models.patient.PatientPutModel;
import es.mdef.mipediatra.models.symptom.SymptomsListModel;
import es.mdef.mipediatra.models.vaccine.VaccinesPersonsListModel;
import es.mdef.mipediatra.repositories.PatientRepository;
import es.mdef.mipediatra.services.AssemblerService;
import es.mdef.mipediatra.services.CrudService;
import es.mdef.mipediatralib.interfaces.IConsultation;
import es.mdef.mipediatralib.interfaces.IEmergency;
import es.mdef.mipediatralib.interfaces.IPatient;
import es.mdef.mipediatralib.interfaces.ISymptom;
import lombok.extern.log4j.Log4j2;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.hateoas.CollectionModel;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.Set;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@Service
@Log4j2
public class PatientsService implements CrudService<PatientEntity, PatientModel, PatientPostModel, PatientPutModel, PatientListModel> {
    private final PatientAssembler patientAssembler;
    private final PatientRepository patientRepository;
    private final VaccinesPersonsService vaccinesPersonsService;
    private final ConsultationService consultationService;
    private final SymptomAssembler symptomAssembler;

    public PatientsService(PatientAssembler patientAssembler, PatientRepository patientRepository, VaccinesPersonsService vaccinesPersonsService, ConsultationService consultationService, SymptomAssembler symptomAssembler) {
        this.patientAssembler = patientAssembler;
        this.patientRepository = patientRepository;
        this.vaccinesPersonsService = vaccinesPersonsService;
        this.consultationService = consultationService;
        this.symptomAssembler = symptomAssembler;
    }

    @Override
    public String getEntityName() {
        return "patient";
    }

    @Override
    public JpaRepository<PatientEntity, Long> getRepository() {
        return patientRepository;
    }

    @Override
    public AssemblerService<PatientEntity, PatientModel, PatientPostModel, PatientPutModel, PatientListModel> getAssembler() {
        return patientAssembler;
    }

    public CollectionModel<VaccinesPersonsListModel> getVaccines(Long id) {
        PatientEntity patient = patientRepository.findById(id).orElseThrow(() -> new EntityNotFoundException(id, "person"));

        if (patient instanceof PersonEntity person) {
            return vaccinesPersonsService.getAssembler()
                    .toCollectionModel(vaccinesPersonsService.findVaccinesPersonsByPerson(person))
                    .removeLinks()
                    .add(linkTo(methodOn(PatientsController.class)
                            .getVaccines(id))
                            .withSelfRel());
        } else {
            throw new PatientNotAPersonException(id);
        }
    }

    public CollectionModel<SymptomsListModel> getSymptoms(Long id, int amount) {
        PatientEntity patient = patientRepository.findById(id).orElseThrow(() -> new EntityNotFoundException(id, "person"));

        Set<EmergencyEntity> consultations = consultationService.getEmergenciesByPatient(patient);
        Set<ISymptom> symptoms = new HashSet<>();

        for (IEmergency e : consultations) {
            if (symptoms.size() < amount) {
                symptoms.addAll(e.getSymptoms());
            } else {
                break;
            }
        }

        Set<SymptomEntity> lastSymptoms = new HashSet<>();
        for (ISymptom symptom : symptoms) {
            if (lastSymptoms.size() < amount) {
                lastSymptoms.add((SymptomEntity) symptom);
            } else {
                break;
            }
        }

        return symptomAssembler.toCollectionModel(lastSymptoms)
                .removeLinks()
                .add(linkTo(methodOn(PatientsController.class)
                        .getSymptoms(id, amount))
                        .withSelfRel());
    }

    public Set<IConsultation> getConsultations(Long id) {
        IPatient patient = patientRepository.findById(id).orElseThrow(() -> new EntityNotFoundException(id, "person"));
        return consultationService.getConsultationsByPatient(patient);
    }

    public void removeConsultationFromPatient(PatientEntity patient, ConsultationEntity consultation) {
        patient.getConsultations().remove(consultation);
        patientRepository.save(patient);
    }
}
